@@ -399,7 +399,7 @@ static PyObject* mod_connect(PyObject* self, PyObject* args, PyObject* kwargs)
     int fAutoCommit = 0;
     int fReadOnly = 0;
     long timeout = 0;
-    Object encoding;
+    PyObject* encoding = 0;
 
     Object attrs_before; // Optional connect attrs set before connecting
 
@@ -865,6 +865,7 @@ static const ConstantDef aConstants[] = {
     MAKECONST(SQL_TYPE_TIME),
     MAKECONST(SQL_TYPE_TIMESTAMP),
     MAKECONST(SQL_SS_TIME2),
+    MAKECONST(SQL_SS_VARIANT),
     MAKECONST(SQL_SS_XML),
     MAKECONST(SQL_INTERVAL_MONTH),
     MAKECONST(SQL_INTERVAL_YEAR),
